%	$Id: icon2symbolic.pl,v 1.2 2008/10/30 01:39:56 mbar098 Exp $

%%  icon2symbolic(IconicProblem, SymbolicProblem)

/*
Icon Meaning of icon 
"M" Beginning of new row 
" " Unoccupied warehouse position 
"$" Box 
"#" Wall 
"@" Warehouse man 
"^" Position outside of warehouse 
"." Elevator 
 
*/
icon2symbolic(IconicRep, problem(SymbolicInitialState, SymbolicGoal)) :-
	(append(IconicRep, NewIconicRep) -> true ; NewIconicRep = IconicRep),
	icon2symbolic(NewIconicRep, [ ], SymbolicInitialState, pos(-1,0), 0, FinalBoxCount),
	list_to_ord_set(SymbolicInitialState, OrdSymbolicInitialState),
	sokobanGoal(FinalBoxCount, OrdSymbolicInitialState, SymbolicGoal).

%% sokobanGoal(Boxes, SymbolicInitialState, SymbolicGoal)
sokobanGoal(Boxes, SymbolicInitialState, SymbolicGoal) :-
	member(elevatorAt(ElevatorPos), SymbolicInitialState),
	sokobanBoxGoals(Boxes, ElevatorPos, [ ], SymbolicGoal).

sokobanBoxGoals(0, _, SymbolicGoal, SymbolicGoal).
sokobanBoxGoals(BoxesLeft, ElevatorPos, OldSymbolicGoal, SymbolicGoal) :-
	BoxesLeft > 0,
	NewBoxesLeft is BoxesLeft - 1,
	sokobanBoxGoals(NewBoxesLeft,
			ElevatorPos,
			[at(box(BoxesLeft), ElevatorPos) | OldSymbolicGoal],
			SymbolicGoal).

%% Icon2symbolic(IconicRep,
%%               OldSymbolicRep,
%%               NewSymbolicRep,
%%               pos(X, Y),
%%               BoxCount,
%%               FinalBoxCount)
%% finished translation
icon2symbolic([], SymbolicRep, SymbolicRep, _, FinalBoxCount, FinalBoxCount).
%% outside of warehouse
icon2symbolic([Char | Rest],
	      OldSymbolicRep,
	      NewSymbolicRep,
	      pos(X,Y),
	      BoxCount,
	      FinalBoxCount) :-
	[Char] = "^",
	NewY is Y + 1,
	Term = outside(pos(X,Y)),
	icon2symbolic(Rest,
		      [Term | OldSymbolicRep],
		      NewSymbolicRep,
		      pos(X, NewY),
		      BoxCount,
		      FinalBoxCount).
icon2symbolic([Char | Rest],
	      OldSymbolicRep,
	      NewSymbolicRep,
	      pos(X,Y),
	      BoxCount,
	      FinalBoxCount) :-
	[Char] = "#",
	NewY is Y + 1,
	Term = wallAt(pos(X,Y)),
	icon2symbolic(Rest,
		      [Term | OldSymbolicRep],
		      NewSymbolicRep,
		      pos(X, NewY),
		      BoxCount,
		      FinalBoxCount).

icon2symbolic([Char | Rest],
	      OldSymbolicRep,
	      NewSymbolicRep,
	      pos(X,Y),
	      BoxCount,
	      FinalBoxCount) :-
	[Char] = ".",
	NewY is Y + 1,
	Term = elevatorAt(pos(X,Y)),
	icon2symbolic(Rest,
		      [Term | OldSymbolicRep],
		      NewSymbolicRep,
		      pos(X, NewY),
		      BoxCount,
		      FinalBoxCount).

%% new row
icon2symbolic([Char | Rest],
	      OldSymbolicRep,
	      NewSymbolicRep,
	      pos(X,_),
	      BoxCount,
	      FinalBoxCount) :-
	[Char] = "M",
	NewX is X +1,
	icon2symbolic(Rest,
		      OldSymbolicRep,
		      NewSymbolicRep,
		      pos(NewX, 0),
		      BoxCount,
		      FinalBoxCount).

icon2symbolic([Char | Rest],
	      OldSymbolicRep,
	      NewSymbolicRep,
	      pos(X, Y),
	      BoxCount,
	      FinalBoxCount) :-
	[Char] = "$",
	NewBoxCount is BoxCount + 1,
	NewY is Y + 1,
	Term = at(box(NewBoxCount), pos(X,Y)),
	icon2symbolic(Rest,
		      [Term |OldSymbolicRep],
		      NewSymbolicRep,
		      pos(X, NewY),
		      NewBoxCount,
		      FinalBoxCount).


icon2symbolic([Char | Rest],
	      OldSymbolicRep,
	      NewSymbolicRep,
	      pos(X, Y),
	      BoxCount,
	      FinalBoxCount) :-
	[Char] = " ",
	NewY is Y + 1,
	icon2symbolic(Rest,
		      OldSymbolicRep,
		      NewSymbolicRep,
		      pos(X, NewY),
		      BoxCount,
		      FinalBoxCount).

%% translating Char into at/2 term 	  
icon2symbolic([Char | Rest],
	      OldSymbolicRep,
	      NewSymbolicRep,
	      pos(X, Y),
	      BoxCount,
	      FinalBoxCount) :-
	[Char] = "@",
	NewY is Y + 1,
	icon2symbolic(Rest,
		      [at(agent, pos(X, Y)) | OldSymbolicRep],
		      NewSymbolicRep,
		      pos(X, NewY),
		      BoxCount,
		      FinalBoxCount).
	