%	$Id: ontology.pl,v 1.4 2008/10/30 01:34:00 mbar098 Exp mbar098 $

/*  predicates can only be 1 of the following:
- derived
- static
- metaLevel
- fluent

Since the default is fluent, all the metaLevel, derived, and static
predicates are described below.

The Ontology:

Name                      Type           Meaning
======================  =========  ============================================ 
at(Thing, Position)      fluent    Thing is at Position

between(P2, P1, P3)      metaLevel position P2 is immediately adjacent
                                   to positions P1 and P3 and all 3 are
				   in a straight (non-diagonal) line

clear(Position)          derived   something can be moved into this
                                   Position

elevatorAt(Position)     static    the elevator is at this Position

neq(A,B)                 metaLevel A and B are not bound to the same
                                   object

nextTo(Pos1, Pos2)       metaLevel Pos1 is adjacent but not diagonal
                                   to Pos2

outside(Position)	 static    this Position is outside the walls

pos(X, Y)                data      position coords X, Y

wallAt(Position)         static    there is a wall at this Position


*/

/*
derived predicates:
====================
clear(Pos) - Pos is inside of warehouse and nothing is at Pos and no
wall is at Pos

*/

%% derived predicates info
:- multifile derived/1.
:- dynamic derived/1.
:- dynamic clear/1.
derived(clear/1).
clear(Pos) :-
	not(outside(Pos)),
	not(wallAt(Pos)),
	not(at(_, Pos)).
/*
clear(Pos) :-
	elevatorAt(Pos).
*/
%% primitive predicates info
:- multifile primitive/1.
:- dynamic primitive/1.
primitive(Pred/Arity) :- not(derived(Pred/Arity)).



/*
static predicates:
==================


*/
:- multifile static/1.
:- dynamic static/1.

:- multifile outside/1.
:- dynamic outside/1.

:- multifile wallAt/1.
:- dynamic wallAt/1.

:- multifile elevatorAt/1.
:- dynamic elevatorAt/1.
static(elevatorAt/1).
static(outside/1).
static(wallAt/1).

%% fluent predicates info
:- multifile fluent/1.
:- dynamic fluent/1.
fluent(Pred/Arity) :- not(static(Pred/Arity)).

/*
  metaLevel predicates:
  =====================
nextTo(Pos1, Pos2)

between(Pos2, Pos1, Pos3) - Pos2 is between Pos1 and Pos3 and Pos1 and*/
/*Pos3 are 2 squares apart

safe(Pos) - it is safe to move a box to this Pos

*/


%% metaLevel predicates info
:- multifile metaLevel/1.
:- dynamic metaLevel/1.

:- multifile neq/2.
:- dynamic neq/2.

%%%% A neq B only A and B cannot be unified
neq(A, B) :- A \== B.

%%%%
:- multifile nextTo/2.
:- dynamic nextTo/2.

nextTo(pos(X, Y1), pos(X, Y2)) :-
	Y2 is Y1 + 1
	;
	Y2 is Y1 -1.
nextTo(pos(X1, Y), pos(X2, Y)) :-
	X2 is X1 + 1
	;
	X2 is X1 - 1.

%%%%%
:- multifile between/3.
:- dynamic between/3.

between(pos(X, Y2), pos(X, Y1), pos(X, Y3)) :-
	nextTo(pos(X, Y2), pos(X, Y1)),
	nextTo(pos(X, Y2), pos(X, Y3)),
	Y1 \== Y3.
between(pos(X2, Y), pos(X1, Y), pos(X3, Y)) :-
	nextTo(pos(X2, Y), pos(X1, Y)),
	nextTo(pos(X2, Y), pos(X3, Y)),
	X1 \== X3.

:- multifile safe/1.
:- dynamic safe/1.

safe(_).

metaLevel(neq/2).
metaLevel(nextTo/2).
metaLevel(between/3).
metaLevel(safe/1).

%% objectLevel predicates info
:- multifile objectLevel/1.
:- dynamic objectLevel/1.
objectLevel(Pred/Arity) :- not(metaLevel(Pred/Arity)).
