/* -*-prolog-*- */
:- multifile programVersion/2.

programVersion(dotGraph, '$Id: dotGraph.pl,v 1.3 2010/08/18 08:37:57 mbar098 Exp mbar098 $').

/*
   Change Log
   ==========
   v1.1: writeDotGraphNode was inadvertantly setting ExpandedOrdinal to nil when
         it was uninstantiated, FIXED

*/

/*  DOT graph file operations
    =========================
*/

openDotGraph(FileName) :-
	atom_codes(FileName, FileNameCodes),
	append(FileNameCodes, ".dot", FullNameCodes),
	atom_codes(FullFileName, FullNameCodes),
	open(FullFileName, write, _DotGraphStream, [alias(dotGraphStream)]),
	write(dotGraphStream, 'digraph '),
	write(dotGraphStream, searchGraph),
	write(dotGraphStream, ' {'),
	format(dotGraphStream, '~ninformation [shape=box,label="Information ~w"];', [FileName]),
	nl(dotGraphStream).

closeDotGraph :-
	write(dotGraphStream, '}'),
	close(dotGraphStream).

writeDotGraphNodeInfo(Node) :-
	nodeName(Node, NodeName),
	nodeState(Node, NodeState),
	nodeFValue(Node, FValue),
	nodeGValue(Node, GValue),
	nodeHValue(Node, HValue),
	nodeExpandedOrdinal(Node, ExpandedOrdinal),
	(var(ExpandedOrdinal) ->
	    Ordinal = nil
	;
	    Ordinal = ExpandedOrdinal),
	format(dotGraphStream, '~N~w [label="~w:~w ~w ~w=~w + ~w"];',
	       [NodeName, NodeName, Ordinal, NodeState, FValue, GValue, HValue]).

writeDotGraphNodeEdgeToParent(Node) :-
	nodeParentState(Node, nil). %% i.e., Node is root of graph
writeDotGraphNodeEdgeToParent(Node) :-
	nodeName(Node, NodeName),
	NodeName \= nil,
	nodeParentState(Node, NodeParentState),
	isClosed(NodeParentState, NodeParent),
	nodeName(NodeParent, ParentName),
	format(dotGraphStream, '~w -> ~w;', [ParentName, NodeName]).
